pub struct Rows { /* private fields */ }Expand description
Results of a prepared statement query.
Implementations
Sourceimpl Rows
impl Rows
Sourcepub fn column_count(&self) -> usize
pub fn column_count(&self) -> usize
Returns the number of columns in the result set.
Sourcepub fn column_name(&self, idx: usize) -> Result<String>
pub fn column_name(&self, idx: usize) -> Result<String>
Returns the name of the column at the given index.
Sourcepub fn column_names(&self) -> Vec<String>
pub fn column_names(&self) -> Vec<String>
Returns the names of all columns in the result set.
Sourcepub fn column_index(&self, name: &str) -> Result<usize>
pub fn column_index(&self, name: &str) -> Result<usize>
Returns the index of the column with the given name.